<?php

namespace EventPortal\Domain\Event\Form\Models;

use EventPortal\Domain\Component\Component;
use EventPortal\Domain\Component\Display;
use EventPortal\Domain\Event\Form\Enums\QuestionType;
use EventPortal\Domain\Model;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\HasMany;
use OwenIt\Auditing\Auditable;
use OwenIt\Auditing\Contracts\Auditable as AuditableInterface;
use Spatie\Enum\Laravel\HasEnums;

class Question extends Model implements AuditableInterface
{
    use HasEnums, Auditable;

    const KEY = 'questions';

    /** @var array<string,mixed> */
    protected $enums = [
        'type' => QuestionType::class,
    ];

    /** @var array<string> */
    protected $casts = [
        'is_sensitive_personal_data' => 'bool',
    ];

    /** @var array<string> */
    protected $fillable = [
        'question',
        'type',
        'help_text',
        'placeholder',
        'form_step_section_id',
        'is_sensitive_personal_data',
        'order',
        'uuid',
    ];

    public function stage(): BelongsTo
    {
        return $this->belongsTo(FormStep::class);
    }

    public function options(): HasMany
    {
        return $this->hasMany(QuestionOption::class)->orderBy('order');
    }

    public function validations(): HasMany
    {
        return $this->hasMany(QuestionValidation::class);
    }

    public function scopeForFormStepSection(Builder $query, FormStepSection $formStepSection): Builder
    {
        return $query->where('form_step_section_id', '=', $formStepSection->id);
    }

    public function isGdprSensitive(): bool
    {
        return $this->is_sensitive_personal_data;
    }

    public function isRequired(): bool
    {
        return $this->validations()->isRequired()->exists();
    }

    public function hasHelpText(): bool
    {
        return $this->help_text !== null;
    }

    public function hasPlaceholder(): bool
    {
        return $this->placeholder !== null;
    }

    public static function typesAsOptions(): array
    {
        $questionTypes = [];

        foreach (QuestionType::getAll() as $type) {
            $questionTypes[] = [
                'label' => $type->getLabel(),
                'value' => $type->getValue(),
            ];
        }

        return $questionTypes;
    }

    /**
     * Return the Question as a component.
     */
    public function asComponent(bool $forDisplay = false): Component
    {
        $class = $forDisplay ? Display::class : $this->type->getBuilderType();

        $component = $class::make($this->question)->scopeTo('formData');

        if ($forDisplay === false) {
            $options = $this->options;

            if ($options->isEmpty() === false) {
                $options = $options
                    ->map
                    ->asOption()
                    ->all();

                $component->options($options);
            }

            $validations = $this->validations;

            if ($validations->isEmpty() === false) {
                $validations = $validations
                    ->map(static function (QuestionValidation $validation): string {
                        return $validation->rule;
                    })
                    ->all();

                $component->rules($validations);
            }
        }

        if ($this->isGdprSensitive()) {
            $component->gdpr();
        }

        if ($this->hasHelpText()) {
            $component->helpText($this->help_text);
        }

        if ($this->hasPlaceholder()) {
            $component->placeholder($this->placeholder);
        }

        return $component;
    }

    public function toArray(): array
    {
        return [
            'id' => $this->id,
            'uuid' => $this->uuid,
            'question' => $this->question,
            'order' => $this->order,
            'hasHelp' => $this->hasHelpText(),
            'helpText' => $this->help_text,
            'isSensitivePersonalData' => $this->is_sensitive_personal_data,
            'required' => $this->validations()->isRequired()->exists(),
            'component' => [
                'name' => [
                    'label' => $this->type->getLabel(),
                    'value' => $this->type,
                ],
                'options' => $this->options->toArray(),
                'placeholder' => $this->placeholder,
            ],
        ];
    }
}
